from garbevents.mock_events import GetData
from garbevents.settings import Settings as ST

"gb -p 8889 -s test_events.py"
ST.url = "https://www.company.com"

ST.mock_json = {"code": "0", "msg": "success", "data": {"demo"}}

addons = [GetData()]



from garbevents.sensors_backstage_events import GetData
from garbevents.settings import Settings as ST

"gb -p 8889 -s test_sensors_backstage_events.py"

ST.url = "/data/logs/sensors/wenwo-cloud-core-domain-doctor-service/"
ST.report_path = "report"
ST.all_events = ["$WebClick", "$pageview"]
ST.events_properties = {
    "graphic_switch": [
        "status_update",
        "doc_set_num",
        "doc_set_price",
        "doc_set_sever_time",
    ]
}
GetData.backstage("server_ip", "server_name", "server_password")

addons = [GetData()]



import tornado.httpserver
import tornado.ioloop
from mitmproxy import addons
from mitmproxy import log
from mitmproxy import master
from mitmproxy import optmanager
from mitmproxy.addons import errorcheck
from mitmproxy.addons import eventstore
from mitmproxy.addons import intercept
from mitmproxy.addons import readfile
from mitmproxy.addons import termlog
from mitmproxy.addons import view
from mitmproxy.contrib.tornado import patch_tornado

from garbevents.mod import maplocals
from garbevents.mod import mapremotes
from garbevents.mod import mockevents
from garbevents.mod import weaknets
from garbevents.web import app
from garbevents.web import static_viewer
from garbevents.web import webaddons


class WebMaster(master.Master):
    def __init__(self, options, with_termlog=True):
        super().__init__(options)
        self.view = view.View()
        self.view.sig_view_add.connect(self._sig_view_add)
        self.view.sig_view_remove.connect(self._sig_view_remove)
        self.view.sig_view_update.connect(self._sig_view_update)
        self.view.sig_view_refresh.connect(self._sig_view_refresh)

        self.events = eventstore.EventStore()
        self.events.sig_add.connect(self._sig_events_add)
        self.events.sig_refresh.connect(self._sig_events_refresh)

        self.options.changed.connect(self._sig_options_update)

        if with_termlog:
            self.addons.add(termlog.TermLog())
        self.addons.add(*addons.default_addons())
        self.addons.add(
            weaknets.WeakNets(),
            mapremotes.MapRemotes(),
            maplocals.MapLocals(),
            mockevents.MockEvents(),
            webaddons.WebAddon(),
            intercept.Intercept(),
            readfile.ReadFile(),
            static_viewer.StaticViewer(),
            self.view,
            self.events,
            errorcheck.ErrorCheck(),
        )
        self.app = app.Application(self, self.options.web_debug)

    def _sig_view_add(self, view, flow):
        app.ClientConnection.broadcast(
            resource="flows", cmd="add", data=app.flow_to_json(flow)
        )

    def _sig_view_update(self, view, flow):
        app.ClientConnection.broadcast(
            resource="flows", cmd="update", data=app.flow_to_json(flow)
        )

    def _sig_view_remove(self, view, flow, index):
        app.ClientConnection.broadcast(resource="flows", cmd="remove", data=flow.id)

    def _sig_view_refresh(self, view):
        app.ClientConnection.broadcast(resource="flows", cmd="reset")

    def _sig_events_add(self, event_store, entry: log.LogEntry):
        app.ClientConnection.broadcast(
            resource="events", cmd="add", data=app.logentry_to_json(entry)
        )

    def _sig_events_refresh(self, event_store):
        app.ClientConnection.broadcast(resource="events", cmd="reset")

    def _sig_options_update(self, options, updated):
        options_dict = optmanager.dump_dicts(options, updated)
        app.ClientConnection.broadcast(
            resource="options", cmd="update", data=options_dict
        )

    async def running(self):
        patch_tornado()
        # Register tornado with the current event loop
        tornado.ioloop.IOLoop.current()

        # Add our web app.
        http_server = tornado.httpserver.HTTPServer(self.app)
        http_server.listen(self.options.web_port, self.options.web_host)

        self.log.info(
            f"Web server listening at http://{self.options.web_host}:{self.options.web_port}/",
        )

        return await super().running()



#!/usr/bin/python
"""
@Author  :  Lijiawei
@Date    :  2022/4/2 4:51 下午
@Desc    :  mitmproxy&garbevents engine.
"""
import argparse
import asyncio
import os
import signal
import sys
import typing

from mitmproxy import exceptions
from mitmproxy import master
from mitmproxy import options
from mitmproxy import optmanager
from mitmproxy.tools import cmdline
from mitmproxy.utils import arg_check
from mitmproxy.utils import debug

from garbevents import __version__


def assert_utf8_env():
    spec = ""
    for i in ["LANG", "LC_CTYPE", "LC_ALL"]:
        spec += os.environ.get(i, "").lower()
    if "utf" not in spec:
        print("Error: mitmproxy requires a UTF console environment.", file=sys.stderr)
        print(
            "Set your LANG environment variable to something like en_US.UTF-8",
            file=sys.stderr,
        )
        sys.exit(1)


def process_options(parser, opts, args):
    if args.version:
        print(debug.dump_system_info())
        sys.exit(0)
    if args.quiet or args.options or args.commands:
        # also reduce log verbosity if --options or --commands is passed,
        # we don't want log messages from regular startup then.
        args.termlog_verbosity = "error"
        args.flow_detail = 0
    if args.verbose:
        args.termlog_verbosity = "debug"
        args.flow_detail = 2

    adict = {}
    for n in dir(args):
        if n in opts:
            adict[n] = getattr(args, n)
    opts.merge(adict)


T = typing.TypeVar("T", bound=master.Master)


def run(
    master_cls: type[T],
    make_parser: typing.Callable[[options.Options], argparse.ArgumentParser],
    arguments: typing.Sequence[str],
    extra: typing.Callable[[typing.Any], dict] = None,
) -> T:  # pragma: no cover
    """
    extra: Extra argument processing callable which returns a dict of
    options.
    """

    async def main() -> T:
        debug.register_info_dumpers()

        opts = options.Options()
        master = master_cls(opts)

        parser = make_parser(opts)

        # To make migration from 2.x to 3.0 bearable.
        if "-R" in sys.argv and sys.argv[sys.argv.index("-R") + 1].startswith("http"):
            print(
                "To use mitmproxy in reverse mode please use --mode reverse:SPEC instead"
            )

        try:
            args = parser.parse_args(arguments)
        except SystemExit:
            arg_check.check()
            sys.exit(1)

        try:
            opts.set(*args.setoptions, defer=True)
            optmanager.load_paths(
                opts,
                os.path.join(opts.confdir, "config.yaml"),
                os.path.join(opts.confdir, "config.yml"),
            )
            process_options(parser, opts, args)

            if args.options:
                optmanager.dump_defaults(opts, sys.stdout)
                sys.exit(0)
            if args.commands:
                master.commands.dump()
                sys.exit(0)
            if extra:
                if args.filter_args:
                    master.log.info(
                        f"Only processing flows that match \"{' & '.join(args.filter_args)}\""
                    )
                opts.update(**extra(args))

        except exceptions.OptionsError as e:
            print(f"{sys.argv[0]}: {e}", file=sys.stderr)
            sys.exit(1)

        loop = asyncio.get_running_loop()

        def _sigint(*_):
            loop.call_soon_threadsafe(
                getattr(master, "prompt_for_exit", master.shutdown)
            )

        def _sigterm(*_):
            loop.call_soon_threadsafe(master.shutdown)

        # We can't use loop.add_signal_handler because that's not available on Windows' Proactorloop,
        # but signal.signal just works fine for our purposes.
        signal.signal(signal.SIGINT, _sigint)
        signal.signal(signal.SIGTERM, _sigterm)

        await master.run()
        return master

    return asyncio.run(main())


def mitmproxy(args=None) -> typing.Optional[int]:  # pragma: no cover
    if os.name == "nt":
        import urwid

        urwid.set_encoding("utf8")
    else:
        assert_utf8_env()
    from mitmproxy.tools import console

    run(console.master.ConsoleMaster, cmdline.mitmproxy, args)
    return None


def mitmdump(args=None) -> typing.Optional[int]:  # pragma: no cover
    logo = r"""                         __                         __      
       ____ _____ ______/ /_  ___ _   _____  ____  / /______
      / __ `/ __ `/ ___/ __ \/ _ \ | / / _ \/ __ \/ __/ ___/
     / /_/ / /_/ / /  / /_/ /  __/ |/ /  __/ / / / /_(__  ) 
     \__, /\__,_/_/  /_.___/\___/|___/\___/_/ /_/\__/____/  v{}
    /____/ """.format(
        __version__
    )
    print(logo)
    from mitmproxy.tools import dump

    def extra(args):
        if args.filter_args:
            v = " ".join(args.filter_args)
            return dict(
                save_stream_filter=v,
                readfile_filter=v,
                dumper_filter=v,
            )
        return {}

    run(dump.DumpMaster, cmdline.mitmdump, args, extra)
    return None


def mitmweb(args=None) -> typing.Optional[int]:  # pragma: no cover
    logo = r"""                         __                         __      
       ____ _____ ______/ /_  ___ _   _____  ____  / /______
      / __ `/ __ `/ ___/ __ \/ _ \ | / / _ \/ __ \/ __/ ___/
     / /_/ / /_/ / /  / /_/ /  __/ |/ /  __/ / / / /_(__  ) 
     \__, /\__,_/_/  /_.___/\___/|___/\___/_/ /_/\__/____/  v{}
    /____/ """.format(
        __version__
    )
    print(logo)
    from garbevents import web

    run(web.master.WebMaster, cmdline.mitmweb, args)
    return None



#!/usr/bin/python
"""
@Author  :  Lijiawei
@Date    :  2022/5/5 1:48 下午
@Desc    :  map local line.
"""
import getpass
import mimetypes
import platform
import re
import typing
import urllib.parse
from pathlib import Path

from mitmproxy import ctx
from mitmproxy import exceptions
from mitmproxy import flowfilter
from mitmproxy import http
from mitmproxy import version
from mitmproxy.utils.spec import parse_spec
from werkzeug.security import safe_join


class MapLocalsSpec(typing.NamedTuple):
    matches: flowfilter.TFilter
    regex: str
    local_path: Path


def parse_map_local_spec(option: str) -> MapLocalsSpec:
    filter, regex, replacement = parse_spec(option)

    try:
        re.compile(regex)
    except re.error as e:
        raise ValueError(f"Invalid regular expression {regex!r} ({e})")

    try:
        if platform.system() == "Darwin":
            default = f"/Users/{getpass.getuser()}/Documents/map_local/"
            path = Path(default + replacement).expanduser().resolve(strict=True)
        else:
            path = Path(replacement).expanduser().resolve(strict=True)
    except FileNotFoundError as e:
        raise ValueError(f"Invalid file path: {replacement} ({e})")

    return MapLocalsSpec(filter, regex, path)


def _safe_path_join(root: Path, untrusted: str) -> Path:
    """Join a Path element with an untrusted str.

    This is a convenience wrapper for werkzeug's safe_join,
    raising a ValueError if the path is malformed."""
    untrusted_parts = Path(untrusted).parts
    joined = safe_join(root.as_posix(), *untrusted_parts)
    if joined is None:
        raise ValueError("Untrusted paths.")
    return Path(joined)


def file_candidates(url: str, spec: MapLocalsSpec) -> list[Path]:
    """
    Get all potential file candidates given a URL and a mapping spec ordered by preference.
    This function already assumes that the spec regex matches the URL.
    """
    m = re.search(spec.regex, url)
    assert m
    if m.groups():
        suffix = m.group(1)
    else:
        suffix = re.split(spec.regex, url, maxsplit=1)[1]
        suffix = suffix.split("?")[0]  # remove query string
        suffix = suffix.strip("/")

    if suffix:
        decoded_suffix = urllib.parse.unquote(suffix)
        suffix_candidates = [decoded_suffix, f"{decoded_suffix}/index.html"]

        escaped_suffix = re.sub(r"[^0-9a-zA-Z\-_.=(),/]", "_", decoded_suffix)
        if decoded_suffix != escaped_suffix:
            suffix_candidates.extend([escaped_suffix, f"{escaped_suffix}/index.html"])
        try:
            return [_safe_path_join(spec.local_path, x) for x in suffix_candidates]
        except ValueError:
            return []
    else:
        return [spec.local_path / "index.html"]


class MapLocals:
    def __init__(self):
        self.replacements: list[MapLocalsSpec] = []

    def load(self, loader):
        loader.add_option(
            "map_locals",
            typing.Sequence[str],
            [],
            """
            使用"|flow-filter|url-regex|file-or-directory-path"形式的模式将远程资源映射到本地文件，其中分隔符是|。
            """,
        )

    def configure(self, updated):
        if "map_locals" in updated:
            self.replacements = []
            for option in ctx.options.map_locals:
                try:
                    spec = parse_map_local_spec(option)
                except ValueError as e:
                    raise exceptions.OptionsError(
                        f"Cannot parse map_locals option {option}: {e}"
                    ) from e

                self.replacements.append(spec)

    def request(self, flow: http.HTTPFlow) -> None:
        if flow.response or flow.error or not flow.live:
            return

        url = flow.request.pretty_url

        all_candidates = []
        for spec in self.replacements:
            if spec.matches(flow) and re.search(spec.regex, url):
                if spec.local_path.is_file():
                    candidates = [spec.local_path]
                else:
                    candidates = file_candidates(url, spec)
                all_candidates.extend(candidates)

                local_file = None
                for candidate in candidates:
                    if candidate.is_file():
                        local_file = candidate
                        break

                if local_file:
                    headers = {"Server": version.MITMPROXY}
                    mimetype = mimetypes.guess_type(str(local_file))[0]
                    if mimetype:
                        headers["Content-Type"] = mimetype

                    try:
                        contents = local_file.read_bytes()
                    except OSError as e:
                        ctx.log.warn(f"Could not read file: {e}")
                        continue

                    flow.response = http.Response.make(200, contents, headers)
                    # only set flow.response once, for the first matching rule
                    return
        if all_candidates:
            flow.response = http.Response.make(404)
            ctx.log.info(
                f"None of the local file candidates exist: {', '.join(str(x) for x in all_candidates)}"
            )



#!/usr/bin/python
"""
@Author  :  Lijiawei
@Date    :  2022/3/29 3:15 下午
@Desc    :  db line.
"""
import os
import sqlite3

from loguru import logger

from garbevents.settings import Settings as ST


class ProxyDbTool:
    """
    ProxyDbTool for sqlite3
    Simple database tool class This class is mainly written to encapsulate sqlite and inherit such reuse methods.
    """

    def __init__(self, file_path=ST.db_path):
        """
        Initialize the database, the default file name proxy.db
        :param file_path:
        """
        if not os.path.exists(ST.db_path):
            os.mkdir(ST.db_path)
        self.filename = file_path + "/" + "proxy.db"
        logger.info(self.filename)
        self.db = sqlite3.connect(self.filename)
        self.c = self.db.cursor()

    def close(self):
        """
        close the database
        :return:
        """
        self.c.close()
        self.db.close()

    def execute(self, sql, param=None):
        """
        Perform database additions, deletions, and changes
        :param sql: sql statement
        :param param: which can be list or tuple, or None
        :return: returns True on success
        """
        try:
            if param is None:
                self.c.execute(sql)
            else:
                if type(param) is list:
                    self.c.executemany(sql, param)
                else:
                    self.c.execute(sql, param)
            count = self.db.total_changes
            self.db.commit()
        except Exception as e:
            print(e)
            return False, e
        if count > 0:
            return True
        else:
            return False

    def query(self, sql, param=None):
        """
        sql query statement
        :param sql: sql statement
        :param param: which can be list or tuple, or None
        :return: returns True on success
        """
        if param is None:
            self.c.execute(sql)
        else:
            self.c.execute(sql, param)
        return self.c.fetchall()


proxy_data = ProxyDbTool()



import base64
import json
import os
import urllib.error
import urllib.parse
import zlib
from pprint import pprint

import mitmproxy
from mitmproxy import ctx

from garbevents.settings import Settings as ST


class GetData:
    """
    A garbevents HTTP request class.
    """

    events_list = []

    @staticmethod
    def chunks(arr, n):
        return [arr[i : i + n] for i in range(0, len(arr), n)]

    def request(self, flow: mitmproxy.http.HTTPFlow):
        """

        :param flow:
        :return:
        """

        request_data = flow.request
        self.request_url = request_data.url
        if ST.url in self.request_url:
            api = self.request_url.split("/")[3].replace("'", "")
            if api in ST.interface_url:
                request_content = (
                    str(flow.request.content)
                    .split("event=")[1]
                    .replace("'", "")
                    .replace(" ", "+")
                )
            else:
                request_content = str(flow.request.url).split("&")[1].split("event=")[1]
            if request_content.find("%") == 0:
                result = urllib.parse.unquote(request_content)
            else:
                url_content = urllib.parse.unquote(request_content)
                ace = base64.b64decode(url_content)
                result = zlib.decompress(ace).decode("utf-8")
            result_list = json.loads(result)
            pprint(result_list)
            try:
                event = result_list["data"][0]["pr"]["$eid"]
                ctx.log.error(
                    f"Get the event name after decrypting the data ====>{event}"
                )
                self.events_list.append(event)
            except KeyError:
                ctx.log.warn("No events！")
            event_list = list(set(self.events_list))

            if not os.path.exists(ST.report_path):
                os.mkdir(ST.report_path)
                ctx.log.info(ST.report_path + "Successfully created！")
            file = open(f"{ST.report_path}/now_event.txt", "w")
            for line in event_list:
                file.write(line + "\n")
            ctx.log.warn(f"Current event name collection ====>{event_list}")
            lost_list = list(set(ST.all_events).difference(set(event_list)))
            ctx.log.warn(f"Missing event name collection ====>{lost_list}")
            file = open(f"{ST.report_path}/lost_event.txt", "w")
            for line in lost_list:
                file.write(line + "\n")



#!/usr/bin/python
"""
@Author  :  Lijiawei
@Date    :  2020/11/20 09:48 上午
@Desc    :  mock line.
"""
import json
import traceback

import mitmproxy
from loguru import logger

from garbevents.settings import Settings as ST


class GetData:
    """
    A garbevents HTTP mock class.
    """

    def __init__(self):
        try:
            self.content = ST.mock["content"]
            self.api = ST.mock["api"]
        except (TypeError, KeyError):
            logger.error(traceback.format_exc())
            self.content = {}
            self.api = ""

    def response(self, flow: mitmproxy.http.HTTPFlow):
        """
        mock modifies the response interface
        :param flow:
        :return:
        """
        if ST.url in flow.request.url:
            if self.api in flow.request.url:
                response = json.loads(flow.response.get_text())
                logger.info(f"original response content: {response}")
                response_content = json.dumps(self.content)
                logger.info(f"mock response content: {self.content}")
                flow.response.set_text(response_content)
                logger.info(f"{self.api} modify success!")



import base64
import gzip
import json
import os
import urllib.error
import urllib.parse
from pprint import pprint

import mitmproxy
from mitmproxy import ctx

from garbevents.settings import Settings as ST


class GetData:
    """
    Personal customized version of 爱问医生
    A engine HTTP request class.
    """

    events_list = []

    @staticmethod
    def chunks(arr, n):
        """

        :param arr:
        :param n:
        :return:
        """
        return [arr[i : i + n] for i in range(0, len(arr), n)]

    @staticmethod
    def gzip_decompress(data):
        """
        解密上报数据
        :param data:
        :return:
        """
        try:
            return gzip.decompress(data)
        except AttributeError:
            from io import StringIO

            buf = StringIO()
            buf.write(data)
            fd = gzip.GzipFile(fileobj=buf, mode="r")
            fd.rewind()
            value = fd.read()
            fd.close()
            return value

    def request(self, flow: mitmproxy.http.HTTPFlow):
        """
        代理服务数据分析
        :param flow:
        :return:
        """

        request_data = flow.request
        self.request_url = request_data.url

        if ST.url in self.request_url:
            ctx.log.info(
                "Get the complete URL after splitting ====>{}".format(
                    self.request_url.split("?")[0]
                )
            )
            api = self.request_url.split("?")[0].split("/")[3]
            ctx.log.error(f"Get API address after splitting ====>{api}")

            # todo: Differentiate between different data sources
            # request_content = str(self.request_url).split('&')[1].split('=')[1]
            request_content = str(flow.request.content).split("&")[0].split("=")[1]
            ctx.log.info(f"Get encrypted data after splitting ====>{request_content}")

            gzip_data = urllib.parse.unquote(request_content)
            data_list = [
                json.loads(base64.b64decode(gzip_data).decode("utf8", errors="ignore"))
            ]

            for result_list in data_list:
                ctx.log.error("Get JSON string after decrypting data ====>")
                pprint(result_list)

                try:
                    event = result_list["event"]
                    ctx.log.error(
                        f"Get the event name after decrypting the data ====>{event}"
                    )
                    self.events_list.append(event)
                except KeyError:
                    ctx.log.warn("No events！")
                event_list = list(set(self.events_list))

                if not os.path.exists(ST.report_path):
                    os.mkdir(ST.report_path)
                    ctx.log.info(ST.report_path + "Successfully created！")

                file = open(f"{ST.report_path}/now_event.txt", "w")
                for line in event_list:
                    file.write(line + "\n")
                ctx.log.warn(f"Current event name collection ====>{event_list}")
                lost_list = list(set(ST.all_events).difference(set(event_list)))
                ctx.log.warn(f"Missing event name collection ====>{lost_list}")
                file = open(f"{ST.report_path}/lost_event.txt", "w")
                for line in lost_list:
                    file.write(line + "\n")



#!/usr/bin/python
"""
@Author  :  Lijiawei
@Date    :  2022/7/12 7:48 下午
@Desc    :  mock events line.
"""
import json
import re
import typing

from mitmproxy import ctx
from mitmproxy import exceptions
from mitmproxy import flowfilter
from mitmproxy import http
from mitmproxy.utils.spec import parse_spec


class MockEventsSpec(typing.NamedTuple):
    matches: flowfilter.TFilter
    subject: str
    replacement: str


def parse_mock_events_spec(option: str) -> MockEventsSpec:
    spec = MockEventsSpec(*parse_spec(option))

    try:
        re.compile(spec.subject)
    except re.error as e:
        raise ValueError(f"Invalid regular expression {spec.subject!r} ({e})")

    return spec


class MockEvents:
    def __init__(self):
        self.replacements: list[MockEventsSpec] = []

    def load(self, loader):
        loader.add_option(
            "mock_events",
            typing.Sequence[str],
            [],
            """
            使用"|flow-filter|url-regex|replacement-json"形式的模式将远程资源映射到本地json，其中分隔符是|。
            """,
        )

    def configure(self, updated):
        if "mock_events" in updated:
            self.replacements = []
            for option in ctx.options.mock_events:
                try:
                    spec = parse_mock_events_spec(option)
                except ValueError as e:
                    raise exceptions.OptionsError(
                        f"Cannot parse mock_events option {option}: {e}"
                    ) from e

                self.replacements.append(spec)

    def request(self, flow: http.HTTPFlow) -> None:
        if flow.response or flow.error or not flow.live:
            return
        url = flow.request.pretty_url
        for spec in self.replacements:
            if spec.matches(flow) and re.search(spec.subject, url):
                try:
                    contents = json.dumps(spec.replacement)
                except (TypeError, KeyError):
                    contents = {}
                flow.response.set_text(contents)



"""
    The following operators are understood:

        ~q          Request
        ~s          Response

    Headers:

        Patterns are matched against "name: value" strings. Field names are
        all-lowercase.

        ~a          Asset content-type in response. Asset content types are:
                        text/javascript
                        application/x-javascript
                        application/javascript
                        text/css
                        image/*
                        font/*
                        application/font-*
        ~h rex      Header line in either request or response
        ~hq rex     Header in request
        ~hs rex     Header in response

        ~b rex      Expression in the body of either request or response
        ~bq rex     Expression in the body of request
        ~bs rex     Expression in the body of response
        ~t rex      Shortcut for content-type header.

        ~d rex      Request domain
        ~m rex      Method
        ~u rex      URL
        ~c CODE     Response code.
        rex         Equivalent to ~u rex
"""
import functools
import re
import sys
from collections.abc import Sequence
from typing import ClassVar
from typing import Protocol
from typing import Union

import pyparsing as pp
from mitmproxy import flow
from mitmproxy import http
from mitmproxy import tcp


def only(*types):
    def decorator(fn):
        @functools.wraps(fn)
        def filter_types(self, flow):
            if isinstance(flow, types):
                return fn(self, flow)
            return False

        return filter_types

    return decorator


class _Token:
    def dump(self, indent=0, fp=sys.stdout):
        print(
            "{spacing}{name}{expr}".format(
                spacing="\t" * indent,
                name=self.__class__.__name__,
                expr=getattr(self, "expr", ""),
            ),
            file=fp,
        )


class _Action(_Token):
    code: ClassVar[str]
    help: ClassVar[str]

    @classmethod
    def make(klass, s, loc, toks):
        return klass(*toks[1:])


class FErr(_Action):
    code = "e"
    help = "Match error"

    def __call__(self, f):
        return True if f.error else False


class FMarked(_Action):
    code = "marked"
    help = "Match marked flows"

    def __call__(self, f):
        return bool(f.marked)


class FHTTP(_Action):
    code = "http"
    help = "Match HTTP flows"

    @only(http.HTTPFlow)
    def __call__(self, f):
        return True


class FWebSocket(_Action):
    code = "websocket"
    help = "Match WebSocket flows"

    @only(http.HTTPFlow)
    def __call__(self, f: http.HTTPFlow):
        return f.websocket is not None


class FTCP(_Action):
    code = "tcp"
    help = "Match TCP flows"

    @only(tcp.TCPFlow)
    def __call__(self, f):
        return True


class FReq(_Action):
    code = "q"
    help = "Match request with no response"

    @only(http.HTTPFlow)
    def __call__(self, f):
        if not f.response:
            return True


class FResp(_Action):
    code = "s"
    help = "Match response"

    @only(http.HTTPFlow)
    def __call__(self, f):
        return bool(f.response)


class FAll(_Action):
    code = "all"
    help = "Match all flows"

    def __call__(self, f: flow.Flow):
        return True


class _Rex(_Action):
    flags = 0
    is_binary = True

    def __init__(self, expr):
        self.expr = expr
        if self.is_binary:
            expr = expr.encode()
        try:
            self.re = re.compile(expr, self.flags)
        except Exception:
            raise ValueError("Cannot compile expression.")


def _check_content_type(rex, message):
    return any(
        name.lower() == b"content-type" and rex.search(value)
        for name, value in message.headers.fields
    )


class FAsset(_Action):
    code = "a"
    help = "Match asset in response: CSS, JavaScript, images, fonts."
    ASSET_TYPES = [
        re.compile(x)
        for x in [
            b"text/javascript",
            b"application/x-javascript",
            b"application/javascript",
            b"text/css",
            b"image/.*",
            b"font/.*",
            b"application/font-.*",
        ]
    ]

    @only(http.HTTPFlow)
    def __call__(self, f):
        if f.response:
            for i in self.ASSET_TYPES:
                if _check_content_type(i, f.response):
                    return True
        return False


class FContentType(_Rex):
    code = "t"
    help = "Content-type header"

    @only(http.HTTPFlow)
    def __call__(self, f):
        if _check_content_type(self.re, f.request):
            return True
        elif f.response and _check_content_type(self.re, f.response):
            return True
        return False


class FContentTypeRequest(_Rex):
    code = "tq"
    help = "Request Content-Type header"

    @only(http.HTTPFlow)
    def __call__(self, f):
        return _check_content_type(self.re, f.request)


class FContentTypeResponse(_Rex):
    code = "ts"
    help = "Response Content-Type header"

    @only(http.HTTPFlow)
    def __call__(self, f):
        if f.response:
            return _check_content_type(self.re, f.response)
        return False


class FHead(_Rex):
    code = "h"
    help = "Header"
    flags = re.MULTILINE

    @only(http.HTTPFlow)
    def __call__(self, f):
        if f.request and self.re.search(bytes(f.request.headers)):
            return True
        if f.response and self.re.search(bytes(f.response.headers)):
            return True
        return False


class FHeadRequest(_Rex):
    code = "hq"
    help = "Request header"
    flags = re.MULTILINE

    @only(http.HTTPFlow)
    def __call__(self, f):
        if f.request and self.re.search(bytes(f.request.headers)):
            return True


class FHeadResponse(_Rex):
    code = "hs"
    help = "Response header"
    flags = re.MULTILINE

    @only(http.HTTPFlow)
    def __call__(self, f):
        if f.response and self.re.search(bytes(f.response.headers)):
            return True


class FBod(_Rex):
    code = "b"
    help = "Body"
    flags = re.DOTALL

    @only(http.HTTPFlow, tcp.TCPFlow)
    def __call__(self, f):
        if isinstance(f, http.HTTPFlow):
            if f.request and f.request.raw_content:
                if self.re.search(f.request.get_content(strict=False)):
                    return True
            if f.response and f.response.raw_content:
                if self.re.search(f.response.get_content(strict=False)):
                    return True
            if f.websocket:
                for msg in f.websocket.messages:
                    if self.re.search(msg.content):
                        return True
        elif isinstance(f, tcp.TCPFlow):
            for msg in f.messages:
                if self.re.search(msg.content):
                    return True
        return False


class FBodRequest(_Rex):
    code = "bq"
    help = "Request body"
    flags = re.DOTALL

    @only(http.HTTPFlow, tcp.TCPFlow)
    def __call__(self, f):
        if isinstance(f, http.HTTPFlow):
            if f.request and f.request.raw_content:
                if self.re.search(f.request.get_content(strict=False)):
                    return True
            if f.websocket:
                for msg in f.websocket.messages:
                    if msg.from_client and self.re.search(msg.content):
                        return True
        elif isinstance(f, tcp.TCPFlow):
            for msg in f.messages:
                if msg.from_client and self.re.search(msg.content):
                    return True


class FBodResponse(_Rex):
    code = "bs"
    help = "Response body"
    flags = re.DOTALL

    @only(http.HTTPFlow, tcp.TCPFlow)
    def __call__(self, f):
        if isinstance(f, http.HTTPFlow):
            if f.response and f.response.raw_content:
                if self.re.search(f.response.get_content(strict=False)):
                    return True
            if f.websocket:
                for msg in f.websocket.messages:
                    if not msg.from_client and self.re.search(msg.content):
                        return True
        elif isinstance(f, tcp.TCPFlow):
            for msg in f.messages:
                if not msg.from_client and self.re.search(msg.content):
                    return True


class FMethod(_Rex):
    code = "m"
    help = "Method"
    flags = re.IGNORECASE

    @only(http.HTTPFlow)
    def __call__(self, f):
        return bool(self.re.search(f.request.data.method))


class FDomain(_Rex):
    code = "d"
    help = "Domain"
    flags = re.IGNORECASE
    is_binary = False

    @only(http.HTTPFlow)
    def __call__(self, f):
        return bool(
            self.re.search(f.request.host) or self.re.search(f.request.pretty_host)
        )


class FUrl(_Rex):
    code = "u"
    help = "URL"
    is_binary = False

    # FUrl is special, because it can be "naked".

    @classmethod
    def make(klass, s, loc, toks):
        if len(toks) > 1:
            toks = toks[1:]
        return klass(*toks)

    @only(http.HTTPFlow)
    def __call__(self, f):
        if not f or not f.request:
            return False
        return self.re.search(f.request.pretty_url)


class FSrc(_Rex):
    code = "src"
    help = "Match source address"
    is_binary = False

    def __call__(self, f):
        if not f.client_conn or not f.client_conn.peername:
            return False
        r = f"{f.client_conn.peername[0]}:{f.client_conn.peername[1]}"
        return f.client_conn.peername and self.re.search(r)


class FDst(_Rex):
    code = "dst"
    help = "Match destination address"
    is_binary = False

    def __call__(self, f):
        if not f.server_conn or not f.server_conn.address:
            return False
        r = f"{f.server_conn.address[0]}:{f.server_conn.address[1]}"
        return f.server_conn.address and self.re.search(r)


class FReplay(_Action):
    code = "replay"
    help = "Match replayed flows"

    def __call__(self, f):
        return f.is_replay is not None


class FReplayClient(_Action):
    code = "replayq"
    help = "Match replayed client request"

    def __call__(self, f):
        return f.is_replay == "request"


class FReplayServer(_Action):
    code = "replays"
    help = "Match replayed server response"

    def __call__(self, f):
        return f.is_replay == "response"


class FMeta(_Rex):
    code = "meta"
    help = "Flow metadata"
    flags = re.MULTILINE
    is_binary = False

    def __call__(self, f):
        m = "\n".join([f"{key}: {value}" for key, value in f.metadata.items()])
        return self.re.search(m)


class FMarker(_Rex):
    code = "marker"
    help = "Match marked flows with specified marker"
    is_binary = False

    def __call__(self, f):
        return self.re.search(f.marked)


class FComment(_Rex):
    code = "comment"
    help = "Flow comment"
    flags = re.MULTILINE
    is_binary = False

    def __call__(self, f):
        return self.re.search(f.comment)


class _Int(_Action):
    def __init__(self, num):
        self.num = int(num)


class FCode(_Int):
    code = "c"
    help = "HTTP response code"

    @only(http.HTTPFlow)
    def __call__(self, f):
        if f.response and f.response.status_code == self.num:
            return True


class FAnd(_Token):
    def __init__(self, lst):
        self.lst = lst

    def dump(self, indent=0, fp=sys.stdout):
        super().dump(indent, fp)
        for i in self.lst:
            i.dump(indent + 1, fp)

    def __call__(self, f):
        return all(i(f) for i in self.lst)


class FOr(_Token):
    def __init__(self, lst):
        self.lst = lst

    def dump(self, indent=0, fp=sys.stdout):
        super().dump(indent, fp)
        for i in self.lst:
            i.dump(indent + 1, fp)

    def __call__(self, f):
        return any(i(f) for i in self.lst)


class FNot(_Token):
    def __init__(self, itm):
        self.itm = itm[0]

    def dump(self, indent=0, fp=sys.stdout):
        super().dump(indent, fp)
        self.itm.dump(indent + 1, fp)

    def __call__(self, f):
        return not self.itm(f)


filter_unary: Sequence[type[_Action]] = [
    FAsset,
    FErr,
    FHTTP,
    FMarked,
    FReplay,
    FReplayClient,
    FReplayServer,
    FReq,
    FResp,
    FTCP,
    FWebSocket,
    FAll,
]
filter_rex: Sequence[type[_Rex]] = [
    # todo: un support filter body
    # FBod,
    # FBodRequest,
    # FBodResponse,
    # FSrc,
    FContentType,
    FContentTypeRequest,
    FContentTypeResponse,
    FDomain,
    FDst,
    FHead,
    FHeadRequest,
    FHeadResponse,
    FMethod,
    FUrl,
    FMeta,
    FMarker,
    FComment,
]
filter_int = [FCode]


def _make():
    # Order is important - multi-char expressions need to come before narrow
    # ones.
    parts = []
    for cls in filter_unary:
        f = pp.Literal(f"~{cls.code}") + pp.WordEnd()
        f.setParseAction(cls.make)
        parts.append(f)

    # This is a bit of a hack to simulate Word(pyparsing_unicode.printables),
    # which has a horrible performance with len(pyparsing.pyparsing_unicode.printables) == 1114060
    unicode_words = pp.CharsNotIn("()~'\"" + pp.ParserElement.DEFAULT_WHITE_CHARS)
    unicode_words.skipWhitespace = True
    regex = (
        unicode_words
        | pp.QuotedString('"', escChar="\\")
        | pp.QuotedString("'", escChar="\\")
    )
    for cls in filter_rex:
        f = pp.Literal(f"~{cls.code}") + pp.WordEnd() + regex.copy()
        f.setParseAction(cls.make)
        parts.append(f)

    for cls in filter_int:
        f = pp.Literal(f"~{cls.code}") + pp.WordEnd() + pp.Word(pp.nums)
        f.setParseAction(cls.make)
        parts.append(f)

    # A naked rex is a URL rex:
    f = regex.copy()
    f.setParseAction(FUrl.make)
    parts.append(f)

    atom = pp.MatchFirst(parts)
    expr = pp.infixNotation(
        atom,
        [
            (pp.Literal("!").suppress(), 1, pp.opAssoc.RIGHT, lambda x: FNot(*x)),
            (pp.Literal("&").suppress(), 2, pp.opAssoc.LEFT, lambda x: FAnd(*x)),
            (pp.Literal("|").suppress(), 2, pp.opAssoc.LEFT, lambda x: FOr(*x)),
        ],
    )
    expr = pp.OneOrMore(expr)
    return expr.setParseAction(lambda x: FAnd(x) if len(x) != 1 else x)


bnf = _make()


class TFilter(Protocol):
    pattern: str

    def __call__(self, f: flow.Flow) -> bool:
        ...  # pragma: no cover


def parse(s: str) -> TFilter:
    """
    Parse a filter expression and return the compiled filter function.
    If the filter syntax is invalid, `ValueError` is raised.
    """
    if not s:
        raise ValueError("Empty filter expression")
    try:
        flt = bnf.parseString(s, parseAll=True)[0]
        flt.pattern = s
        return flt
    except (pp.ParseException, ValueError) as e:
        raise ValueError(f"Invalid filter expression: {s!r}") from e


def match(flt: Union[str, TFilter], flow: flow.Flow) -> bool:
    """
    Matches a flow against a compiled filter expression.
    Returns True if matched, False if not.

    If flt is a string, it will be compiled as a filter expression.
    If the expression is invalid, ValueError is raised.
    """
    if isinstance(flt, str):
        flt = parse(flt)
    if flt:
        return flt(flow)
    return True


match_all: TFilter = parse("~all")
"""A filter function that matches all flows"""

help = []
for a in filter_unary:
    help.append((f"~{a.code}", a.help))
for b in filter_rex:
    help.append((f"~{b.code} regex", b.help))
for c in filter_int:
    help.append((f"~{c.code} int", c.help))
help.sort()
help.insert(0, (f"~{FSrc.code} regex", FSrc.help))
help.extend(
    [
        ("!", "unary not"),
        ("&", "and"),
        ("|", "or"),
        ("(...)", "grouping"),
    ]
)



#!/usr/bin/python
"""
@Author  :  Lijiawei
@Date    :  2022/3/22 5:45 下午
@Desc    :  toolbox line.
"""
import ast
import datetime
import getpass
import json
import os
import platform
import time
import traceback
import urllib.parse
from pathlib import Path

import jmespath
import requests
from airtest.core.api import assert_equal
from airtest.core.api import log
from deepdiff import DeepDiff
from loguru import logger

from garbevents.settings import Settings as ST


def diff(expect, actual, complete=True, exclude_paths=None, view="text") -> dict:
    """
    analysis diff result
    old new
    :param view: tree or text or _delta
    :param expect: expect
    :param actual: actual
    :param complete: match type, match complete or include, default complete.
    :param exclude_paths: parameter whitelist is empty by default.
    :return: {'result': True, 'data': {}}
    :Example:
        >>> {'result': True, 'data': {'dictionary_item_added': [root['data']]}}
    """
    exclude_paths = {f"root['{path}']" for path in str(exclude_paths).split(",")}
    if len(exclude_paths) == 0:
        exclude_paths = None

    if not isinstance(expect, dict):
        expect = ast.literal_eval(expect)
    if not isinstance(actual, dict):
        actual = ast.literal_eval(actual)

    log(arg=expect, desc="Expect")
    log(arg=actual, desc="Actual")

    compare_results = DeepDiff(
        expect, actual, view=view, ignore_order=True, exclude_paths=exclude_paths
    ).to_dict()
    if not complete:
        if compare_results.get("values_changed"):
            result = False
        else:
            result = True
    else:
        if (
            compare_results.get("dictionary_item_added")
            or compare_results.get("dictionary_item_removed")
            or compare_results.get("values_changed")
            or compare_results.get("iterable_item_added")
            or compare_results.get("iterable_item_removed")
        ):
            result = False
        else:
            result = True

    return {"result": result, "data": compare_results}


def extract(expressions, data):
    """
    extract the value you want！
    help documentation >>> https://jmespath.org/tutorial.html
    :param expressions: jsonpath expressions
    :param data: data dict
    :return: the value you want
    :Example:
        >>> test_dict = {"a": {"b": {"c": {"d": "value"}}}}
        >>> result = extract(expressions='a.b.c.d', data = test_dict)
        >>> print(result)
        >>> # value
    """
    _data = jmespath.search(expressions, data)
    return _data


def timestamp():
    """
    timestamp tool
    :return:
    """
    return int(round(time.time() * 1000))


def datetime_strife(fmt="%Y-%m-%d_%H-%M-%S"):
    """
    format datetime tool
    :param fmt: %Y-%m-%d_%H-%M-%S
    :return:
    """
    return datetime.datetime.now().strftime(fmt)


def sleep(seconds=1.0):
    """
    sleep time
    :param seconds:
    :return:
    """
    time.sleep(seconds)


def target(target_url, sign="stag=", index=1) -> dict:
    """
    the value you want！
    :param index: default 1
    :param sign: split flag
    :param target_url: encoded string
    :return: the value you want！
    """
    stag = ast.literal_eval(
        str(urllib.parse.unquote(target_url).split(sign)[index].split("&")[0])
    )

    return stag


def assert_diff(result: dict, equal: bool = True):
    """
    Assert two values are equal
    :param equal: verify True or False, default True
    :param result: toolbox.diff() return value
    :raise AssertionError: if assertion
    :return: None
    """
    # Report step
    log(
        arg=str(result["data"])
        .replace("old_value", "expect_value")
        .replace("new_value", "actual_value"),
        desc="Diff Details",
    )

    # Assertion
    try:
        assert_equal(first=equal, second=result["result"], msg="Diff Result")
    except AssertionError:
        pass


def search(start_time, end_time, api, types):
    """

    :param types: request or response
    :param start_time: 13-bit, millisecond timestamp, 1648545731537.
    :param end_time: default current timestamp, 1648645731537
    :param api: interface name.
    :return: interface information list.
    :Example:
        >>> search_data = search(start_time=1648545731537, end_time=1648645731537, api='/api/test', types='request')
        >>> print(search_data)
        >>> # [{'invokedBy': 'open'}]
    """
    from garbevents.db import proxy_data

    data_collection = proxy_data.query(
        f"select {types} from proxy where timestamps between {start_time} and {end_time} "
        f"and api == '{api}';"
    )
    # todo 去掉ip校验
    #  and client_ip == '{device().get_ip_address()}'

    res = []
    for data in data_collection:
        for d in data:
            res.append(ast.literal_eval(d))
    return res


def str_to_bool(value):
    """
    str convert bool
    :param value:
    :return:
    """
    return True if value.lower() == "true" else False


def match(cases: list, sign="stag=", index=1, equal: bool = True):
    """

    :param cases:
    :param sign:
    :param index:
    :param equal:
    :raise AssertionError: if assertion
    :return: None
    """
    for info in cases:
        if not isinstance(info, dict):
            info = ast.literal_eval(info)
        try:
            excepts = info["excepts"]

            search_data = search(
                start_time=info["search"]["start_time"],
                end_time=info["search"]["end_time"],
                api=info["api"],
                types=info["types"],
            )
            detail = []
            for _data in search_data:
                actual = extract(expressions=info["extract"], data=_data)

                if str_to_bool(info["target"]):
                    actual = target(target_url=actual, sign=sign, index=index)

                result = diff(
                    expect=excepts,
                    actual=actual,
                    complete=str_to_bool(info["diff"]["complete"]),
                    exclude_paths=info["diff"]["exclude_paths"],
                )
                detail.append(result)

            if len(detail) > 1:
                flag = True
                for res in detail:
                    if res["result"]:
                        assert_diff(result=res, equal=equal)
                        flag = False
                        break
                if flag:
                    log(
                        arg=str(detail)
                        .replace("old_value", "expect_value")
                        .replace("new_value", "actual_value"),
                        desc="No Results Were Matched",
                    )
                    assert_equal(first=True, second=False, msg="Multiple Diff Result")
            else:
                try:
                    assert_diff(result=detail[0], equal=equal)
                except IndexError:
                    logger.error(traceback.format_exc())
        except AssertionError:
            logger.error(traceback.format_exc())


def mock(options: list, events=ST.options):
    """
    mock api
    :param events: mock key
    :param options: {"api": "", "content": ""}
    :return: {"map_locals": ["|flow-filter|url-regex|file-or-directory-path"]}
    """
    mock_opt = []
    for option in options:
        try:
            url_regex = option["api"]
            replacement_json = option["content"]
        except (TypeError, KeyError):
            logger.error(traceback.format_exc())
            url_regex = ""
            replacement_json = ""

        replacement = str(url_regex).replace("/", "_") + ".json"
        if platform.system() == "Darwin":
            path = f"/Users/{getpass.getuser()}/Documents/map_local/{replacement}"
        else:
            path = os.path.join(Path().expanduser().resolve(strict=True), replacement)

        with open(path, "w") as f:
            json.dump(replacement_json, f, ensure_ascii=False)

        mock_opt.append(f"|{url_regex}|{replacement}")

        log(arg=option, desc="Mock Details")

    url_put = f"http://{ST.web_host}:{ST.web_port}/options"
    headers_put = {"Content-Type": "application/json"}
    param = {events: mock_opt}
    payload = json.dumps(param)
    try:
        requests.put(url_put, data=payload, headers=headers_put)
    except Exception:
        logger.error("ConnectionRefusedError or ConnectionError")



#!/usr/bin/python
"""
@Author  :  Lijiawei
@Date    :  2022/3/25 5:03 下午
@Desc    :  test_capture line.
"""
from garbevents.capture import GetData
from garbevents.settings import Settings as ST

"gb -p 8888 --ssl-insecure -s test_capture.py"

ST.url = "https://test.com"

ST.db_path = "proxy"

addons = [GetData()]



#!/usr/bin/env python
from setuptools import find_packages
from setuptools import setup

from garbevents import __version__

with open("README.md", encoding="utf-8") as f:
    long_description = f.read()

setup(
    name="garbevents",
    version=__version__,
    keywords=[
        "pip",
        "garbevents",
        "zhugeio",
        "buried points",
        "sensors",
        "argo",
        "growingio",
    ],
    description="grabbing buried points tools.",
    long_description=long_description,
    long_description_content_type="text/markdown",
    license="MIT Licence",
    url="https://github.com/Pactortester/garbevents",
    author="lijiawei",
    author_email="jiawei.li2@qq.com",
    classifiers=[
        "License :: OSI Approved :: MIT License",
        "Development Status :: 5 - Production/Stable",
        "Environment :: Console :: Curses",
        "Operating System :: MacOS",
        "Operating System :: POSIX",
        "Operating System :: Microsoft :: Windows",
        "Programming Language :: Python :: 3 :: Only",
        "Programming Language :: Python :: 3.8",
        "Programming Language :: Python :: 3.9",
        "Programming Language :: Python :: 3.10",
        "Programming Language :: Python :: Implementation :: CPython",
        "Topic :: Security",
        "Topic :: Internet :: WWW/HTTP",
        "Topic :: Internet :: Proxy Servers",
        "Topic :: System :: Networking :: Monitoring",
        "Topic :: Software Development :: Testing",
        "Typing :: Typed",
    ],
    packages=find_packages(
        include=[
            "garbevents",
            "garbevents.*",
        ]
    ),
    include_package_data=True,
    entry_points={
        "console_scripts": [
            "gb = garbevents.cli.main:mitmdump",
            "gbweb = garbevents.cli.main:mitmweb",
        ]
    },
    python_requires=">=3.8",
    platforms="any",
    install_requires=[
        "mitmproxy==8.0.0",
        "jsonpath",
        "paramiko",
        "emoji",
        "deepdiff",
        "jmespath",
        "loguru",
    ],
)



__version__ = "2.1.7"
__description__ = (
    f"grabbing buried points tools. Created: Lijiawei. Version {__version__}"
)
"""
This is a python toolkit for real-time capture,
analysis, cleaning and report generation of embedded points based on the development of mitmproxy.

"""



import gzip
import json
import os
import select
from collections import Counter
from datetime import datetime
from pprint import pprint

import jsonpath
import paramiko
from mitmproxy import ctx

from garbevents.settings import Settings as ST


class GetData:
    """
    Capture of buried points in the background.
    A engine HTTP request class.
    """

    events_list = []
    now_time = datetime.now()
    new_time = now_time.strftime("%Y-%m-%d %H:%M:%S")

    @staticmethod
    def chunks(arr, n):
        """

        :param arr:
        :param n:
        :return:
        """
        return [arr[i : i + n] for i in range(0, len(arr), n)]

    @staticmethod
    def gzip_decompress(data):
        """
        解密上报数据
        :param data:
        :return:
        """
        try:
            return gzip.decompress(data)
        except AttributeError:
            from io import StringIO

            buf = StringIO()
            buf.write(data)
            fd = gzip.GzipFile(fileobj=buf, mode="r")
            fd.rewind()
            value = fd.read()
            fd.close()
            return value

    @staticmethod
    def backstage(serverip, user, pwd):
        """

        :param serverip:
        :param user:
        :param pwd:
        :return:
        """
        events_list = []
        # 构造当天的日志文件名字，根据不同的埋点名称再做修改
        now_time = datetime.now()
        new_time = now_time.strftime("%Y-%m-%d")
        log = "sensors.log." + new_time

        # 进行连接
        ctx.log.info("------------开始连接服务器(%s)-----------" % serverip)
        client = paramiko.SSHClient()
        client.set_missing_host_key_policy(paramiko.AutoAddPolicy())
        ctx.log.info("------------开始认证......-----------")
        client.connect(serverip, 22, username=user, password=pwd, timeout=4)
        ctx.log.info("------------认证成功!.....-----------")
        # 开启channel 管道
        transport = client.get_transport()
        channel = transport.open_session()
        channel.get_pty()
        # 执行命令nohup.log.2017-12-30
        tail = "tail -n 0 -f  " + ST.url + log
        # 将命令传入管道中
        channel.exec_command(tail)
        while True:
            # 用于精确查出什么时间时候没有值
            news_time = now_time.strftime("%Y-%m-%d %H:%M:%S")

            # 判断退出的准备状态
            if channel.exit_status_ready():
                break
            try:
                # 通过socket进行读取日志，linux相当于客户端
                rl, wl, el = select.select([channel], [], [])
                if len(rl) > 0:
                    recv = channel.recv(1024)
                    # 此处将获取的数据解码成utf-8的读取
                    AAA = recv.decode("utf-8", "ignore")
                    BBB = AAA.split("\n")
                    DDD = BBB[0:2]
                    for CCC in DDD:
                        try:
                            GGG = str(CCC)
                            data = json.loads(GGG)
                            try:
                                event = data["event"]
                                if event in ST.all_events:
                                    ctx.log.info(f"【后端】获取到的埋点事件名称是 ====>{event}")
                                    events_list.append(event)
                                    count = Counter(events_list)
                                    lost_list = list(
                                        set(ST.all_events).difference(set(events_list))
                                    )

                                    # 用于遍历对应event事件里面的具体信息
                                    for event_value in ST.events_properties[event]:
                                        # 记录下坑，下面不能直接在里面传event_value不然取不到值
                                        result = jsonpath.jsonpath(
                                            data, f"$..{event_value}"
                                        )
                                        if result is False:
                                            ctx.log.info(
                                                f"【后端】请注意！！！请注意！！！{event}事件未获取到{event_value}值"
                                            )
                                            with open(
                                                f"{ST.report_path}/【后端】埋点事件参数没有值的统计.txt",
                                                "a+",
                                                encoding="utf-8",
                                            ) as file01:
                                                # 用于记录埋点事件参数没有值的统计.txt
                                                file01.write(
                                                    "【后端】{}请注意！！！请注意！！！{}事件未获取到{}值!!!".format(
                                                        news_time, event, event_value
                                                    )
                                                    + "\n"
                                                )
                                        else:
                                            ctx.log.info(
                                                f"【后端】获取到的埋点事件:{event}的自定义参数{event_value}的值是{result}："
                                            )

                                    # 在已上报的埋点事件统计.txt记录已经获取到的事件
                                    with open(
                                        f"{ST.report_path}/【后端】已上报的埋点事件统计.txt",
                                        "w",
                                        encoding="utf-8",
                                    ) as file03:
                                        # file03.writelines(events_list)
                                        for line in events_list:
                                            file03.write(line + "\n")

                                    # 在漏上报的埋点事件统计.txt记录已经获取到的事件
                                    with open(
                                        f"{ST.report_path}/【后端】漏上报的埋点事件统计.txt",
                                        "w",
                                        encoding="utf-8",
                                    ) as file04:
                                        # file04.writelines(lost_list)
                                        for lines in lost_list:
                                            file04.write(lines + "\n")
                                            ctx.log.info(
                                                f"【后端】暂未获取到这些埋点事件，可能存在漏上报事件 ====>{lines}"
                                            )

                                    ctx.log.info(f"【后端】累计的埋点事件统计次数{count}")
                                    with open(
                                        f"{ST.report_path}/【后端】累计的埋点事件统计次数.txt",
                                        "w",
                                        encoding="utf-8",
                                    ) as file02:
                                        # 用于记录埋点事件参数没有值的统计.txt
                                        file02.write(f"【后端】累计统计到的埋点次数{count}!!!" + "\n")

                                    # 不存在对应txt文件时，先创建txt文件
                                    if not os.path.exists(ST.report_path):
                                        os.mkdir(ST.report_path)
                                        ctx.log.info(
                                            ST.report_path + "Successfully created！"
                                        )
                                else:
                                    ctx.log.info(f"该事件{event}不在你录的事件中")
                            except Exception:
                                ctx.log.info("json中不存在event字段,请查看上面具体json查询想要的内容字段")
                                pprint(f"json中不存在event字段{data}")
                                break
                        except json.decoder.JSONDecodeError as e01:
                            pass
                        except Exception as e02:
                            print(e02)
                            ctx.log.info("读取不到日志文件，还未生成，请先手动触发事件生成日志")

            # 键盘终端异常
            except KeyboardInterrupt:
                print("Caught control-C")
                channel.send("\x03")  # 发送 ctrl+c
                channel.close()
        client.close()
